"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateReceivedFiles = exports.destination = exports.getFiles = exports.initFiles = void 0;
const fs_1 = __importDefault(require("fs"));
const readline_1 = __importDefault(require("readline"));
const path_1 = __importDefault(require("path"));
const fileHandler = fs_1.default.promises;
let directFiles = [];
let listFiles;
let receivedFiles = [];
let destination;
exports.destination = destination;
function isPathPresent(arr, path) {
    return arr.find(value => value.path === path);
}
function addPath(path) {
    if (!isPathPresent(directFiles, path) && !isPathPresent(listFiles, path))
        listFiles.push(fileMaker(path));
}
function readLines(listPath) {
    return new Promise((resolve, reject) => {
        const readInterface = readline_1.default.createInterface({
            input: fs_1.default.createReadStream(listPath),
            terminal: false,
        });
        readInterface.on('close', () => {
            resolve();
            calcSize(listFiles);
        });
        let lineNumber = 1;
        readInterface.on('line', line => {
            if (fs_1.default.existsSync(line))
                addPath(line);
            else
                reject(`${line} does not exist! (at ${path_1.default.resolve(process.cwd(), listPath)} at ${lineNumber})`);
            lineNumber++;
        });
    });
}
function getSize(givenPath) {
    return new Promise((resolve, reject) => {
        fileHandler.stat(givenPath).then(stats => {
            if (!stats.isDirectory())
                setImmediate(resolve, stats.size);
            else
                fileHandler
                    .readdir(givenPath)
                    .then(children => children.map(child => path_1.default.resolve(givenPath, child)))
                    .then(paths => Promise.all(paths.map(getSize)))
                    .then(sizes => sizes.reduce((total, size) => total + size, stats.size))
                    .then(resolve);
        });
    });
}
function calcSize(arr, index = 0) {
    if (arr[index])
        getSize(arr[index].path).then(size => {
            arr[index].size = size;
            calcSize(arr, ++index);
        });
}
function fileMaker(filePath) {
    return {
        path: filePath,
        size: undefined,
        folder: fs_1.default.existsSync(filePath) && fs_1.default.statSync(filePath).isDirectory(),
        originalname: path_1.default.basename(filePath),
        filename: path_1.default.basename(filePath),
    };
}
function initFiles(options) {
    if (!Array.isArray(options.files))
        options.files = [];
    directFiles = options.files
        .filter((value, index, self) => self.indexOf(value) === index)
        .map(fileMaker);
    exports.destination = destination =
        options.destination ||
            path_1.default.resolve(path_1.default.basename(require.main.filename), '../uploads');
    calcSize(directFiles);
    fs_1.default.access(destination, fs_1.default.constants.F_OK, err => {
        if (err)
            fs_1.default.mkdirSync(destination, { recursive: true });
    });
    listFiles = [];
    if (options.restart)
        receivedFiles = [];
    return new Promise((resolve, reject) => {
        if (options.list)
            readLines(options.list)
                .then(resolve)
                .catch(err => reject(err));
        else
            resolve();
    });
}
exports.initFiles = initFiles;
function getFiles() {
    return directFiles.concat(listFiles, receivedFiles);
}
exports.getFiles = getFiles;
function updateReceivedFiles(uploadedFiles) {
    uploadedFiles.forEach(function (value) {
        if (!isPathPresent(receivedFiles, value.path)) {
            receivedFiles.push(value);
        }
    });
}
exports.updateReceivedFiles = updateReceivedFiles;
//# sourceMappingURL=fileManager.js.map