"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateReceivedFiles = exports.saveZips = exports.saveFiles = void 0;
const busboy_1 = __importDefault(require("busboy"));
const unzip_stream_1 = __importDefault(require("unzip-stream"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const fileManager_1 = require("./fileManager");
Object.defineProperty(exports, "updateReceivedFiles", { enumerable: true, get: function () { return fileManager_1.updateReceivedFiles; } });
function uniqueName(name, folder) {
    const parent = fileManager_1.destination;
    let number = 0;
    let filename;
    do
        filename = `${path_1.default.basename(name, path_1.default.extname(name))}${number-- || ''}${folder ? '' : path_1.default.extname(name)}`;
    while (fs_1.default.existsSync(path_1.default.resolve(parent, filename)));
    return {
        path: path_1.default.resolve(parent, filename),
        originalname: name,
        filename: filename,
        size: -1,
        folder: folder,
    };
}
function fileWriter(filePath, fileStream) {
    return new Promise((resolve, reject) => {
        fs_1.default.mkdirSync(path_1.default.dirname(filePath), {
            recursive: true,
        });
        const destStream = fs_1.default
            .createWriteStream(filePath)
            .on('close', () => {
            resolve(destStream.bytesWritten);
        })
            .on('error', reject);
        fileStream.pipe(destStream);
    });
}
function extracter(destination, zipstream) {
    return new Promise((resolve, reject) => {
        let size = 0;
        let entries = 0;
        const unzipper = unzip_stream_1.default
            .Parse()
            .on('error', reject)
            .on('entry', (entry) => {
            entries++;
            if (entry.type === 'Directory') {
                entries--;
                fs_1.default.mkdirSync(path_1.default.resolve(destination, entry.path), {
                    recursive: true,
                });
            }
            else {
                fileWriter(path_1.default.resolve(destination, entry.path), entry).then((bytesWritten) => {
                    size = size + bytesWritten;
                    if (--entries === 0)
                        resolve(size);
                });
            }
        });
        zipstream.pipe(unzipper);
    });
}
function saveFiles(request, _response, next) {
    if (request.method !== 'POST' || !request.headers['content-type'])
        next();
    else {
        let writer = new busboy_1.default({ headers: request.headers });
        writer.on('file', (_fieldname, file, filename, _encoding, _mimetype) => {
            request.files = request.files || [];
            let fileObject = uniqueName(filename, false);
            request.files.push(fileObject);
            let writeStream = fs_1.default.createWriteStream(fileObject.path);
            writeStream.on('close', () => {
                request.files.find(file => file.path === fileObject.path).size = writeStream.bytesWritten;
                if (!request.files.filter(file => file.size === -1).length)
                    next();
            });
            file.pipe(writeStream);
        });
        writer.on('finish', () => {
            if (!request.files)
                next();
        });
        request.pipe(writer);
    }
}
exports.saveFiles = saveFiles;
function saveZips(request, _response, next) {
    if (request.method !== 'POST' || !request.headers['content-type'])
        next();
    else {
        let writer = new busboy_1.default({ headers: request.headers });
        writer.on('file', (_fieldname, file, filename, _encoding, mimetype) => {
            if (mimetype === 'application/zip' ||
                mimetype === 'application/x-zip-compressed') {
                request.files = request.files || [];
                let folderObject = uniqueName(filename, true);
                request.files.push(folderObject);
                extracter(folderObject.path, file).then(size => {
                    request.files.find(file => file.path === folderObject.path).size = size;
                    if (!request.files.filter(file => file.size === -1).length)
                        next();
                });
            }
            else
                file.resume();
        });
        writer.on('finish', () => {
            if (!request.files)
                next();
        });
        request.pipe(writer);
    }
}
exports.saveZips = saveZips;
//# sourceMappingURL=uploadManager.js.map