"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.router = void 0;
const express_1 = __importDefault(require("express"));
const downloadManager_1 = require("../middleware/downloadManager");
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const router = express_1.default.Router();
exports.router = router;
router.all('/list', (request, response, next) => {
    if (request.method === 'GET') {
        response.json(downloadManager_1.fileList());
    }
    else {
        const error = new Error('Only GET requests are allowed on this route');
        error.status = 400;
        return next(error);
    }
});
router.all('/specific', (request, response, next) => {
    if (request.method === 'GET') {
        const index = request.query.index;
        const indices = index ? index.split(',') : undefined;
        const fileArray = downloadManager_1.getFiles();
        if (indices && indices.every(index => fileArray[index])) {
            if (indices.length === 1 && !fileArray[indices[0]].folder)
                response.download(fileArray[indices[0]].path);
            else {
                const archive = archiver_1.default('zip', {
                    store: true,
                });
                archive.on('error', err => {
                    const error = new Error(err.message);
                    error.status = 500;
                    return next(error);
                });
                response.attachment(indices.length === 1
                    ? `${path_1.default.basename(fileArray[indices[0]].path)}.zip`
                    : 'variousFiles.zip');
                archive.pipe(response);
                indices.map(index => {
                    const file = fileArray[index];
                    file.folder
                        ? archive.directory(file.path, path_1.default.basename(file.path))
                        : archive.append(file.path, {
                            name: path_1.default.basename(file.path),
                        });
                });
                archive.finalize();
            }
        }
        else if (fileArray.length === 0) {
            const error = new Error('There are no files for download!');
            error.status = 403;
            return next(error);
        }
        else {
            const error = new Error('Wrong data supplied!');
            error.status = 400;
            return next(error);
        }
    }
    else {
        const error = new Error('Only GET requests are allowed on this route');
        error.status = 400;
        return next(error);
    }
});
router.all('/all', (request, response, next) => {
    if (request.method === 'GET') {
        const fileArray = downloadManager_1.getFiles();
        if (fileArray.length > 0) {
            const archive = archiver_1.default('zip', {
                store: true,
            });
            archive.on('error', err => {
                const error = new Error(err.message);
                error.status = 500;
                return next(error);
            });
            archive.pipe(response);
            response.attachment('allFiles.zip');
            fileArray.map(file => {
                file.folder
                    ? archive.directory(file.path, path_1.default.basename(file.path))
                    : archive.append(file.path, {
                        name: path_1.default.basename(file.path),
                    });
            });
            archive.finalize();
        }
        else {
            const error = new Error('There are no files for download!');
            error.status = 403;
            return next(error);
        }
    }
    else {
        const error = new Error('Only GET requests are allowed on this route');
        error.status = 400;
        return next(error);
    }
});
//# sourceMappingURL=download.js.map